import mongoengine

HASHLINK_REGEX = r'[A-Za-z0-9]+'

class HashlinkDocument(mongoengine.Document):
    state = mongoengine.DictField(required=True, unique=True)
    hashlink = mongoengine.StringField(required=True, unique=True, regex=HASHLINK_REGEX, min_length=8, max_length=8)

class HashlinkPathDocument(mongoengine.Document):
    previous = mongoengine.StringField(required=False, regex=HASHLINK_REGEX, min_length=8, max_length=8)
    current = mongoengine.StringField(required=True, regex=HASHLINK_REGEX, min_length=8, max_length=8)
    session_id = mongoengine.StringField(required=True)



#!/usr/bin/env python

import os

from setuptools import setup, find_packages

try:
    # Workaround for http://bugs.python.org/issue15881
    import multiprocessing
except ImportError:
    pass

VERSION = '0.1'

if __name__ == '__main__':
    setup(
        name = 'django-hashlink',
        version = VERSION,
        description = "Store and retrieve URL hash (fragment) based webpage state in MongoDB through Django.",
        long_description = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
        author = 'Mitar',
        author_email = 'mitar.django@tnode.com',
        url = 'https://github.com/mitar/django-hashlink',
        license = 'AGPLv3',
        packages = find_packages(exclude=('*.tests', '*.tests.*', 'tests.*', 'tests')),
        package_data = {},
        classifiers = [
            'Development Status :: 4 - Beta',
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: GNU Affero General Public License v3',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Framework :: Django',
        ],
        include_package_data = True,
        zip_safe = False,
        install_requires = [
            'Django>=1.3',
            'mongoengine>=0.6.11',
        ],
    )



from django import http
from django.core import signing
from django.utils import crypto, simplejson

from . import models

SESSION_ID_SALT = 'django-hashlink'

def hashlink_view(request):
    def store_path(previous, hashlink):
        if previous != hashlink:
            session_id = signing.base64_hmac(SESSION_ID_SALT, str(request.user.id or request.session.session_key), None)
            models.HashlinkPathDocument.objects.create(previous=previous, current=hashlink, session_id=session_id)
        else:
            # TODO: Log, redundant
            # TODO: Remove
            print "redundant call"
            pass

    if not request.method == 'POST':
        # TODO: Log
        return http.HttpResponseBadRequest()

    # TODO: Remove
    print request.POST

    previous = request.POST.get('previous') or None
    if previous and not models.HashlinkDocument.objects.filter(hashlink=previous).count():
        # TODO: Log
        return http.HttpResponseBadRequest()

    if 'state' in request.POST:
        try:
            state = simplejson.loads(request.POST['state'])
        except ValueError:
            # TODO: Log
            return http.HttpResponseBadRequest()

        if not state:
            # TODO: Log
            return http.HttpResponseBadRequest()

        # TODO: Remove race condition
        try:
            hashlink = models.HashlinkDocument.objects.only('hashlink').get(state=state)
        except models.HashlinkDocument.DoesNotExist:
            # TODO: Improve with database retry if non-unique
            hashlink = models.HashlinkDocument.objects.create(state=state, hashlink=crypto.get_random_string(8))

        store_path(previous, hashlink.hashlink)

        # TODO: Remove
        print hashlink.hashlink

        return http.HttpResponse(simplejson.dumps({
            'hashlink': hashlink.hashlink,
        }), mimetype="application/json; charset=utf-8")

    elif 'hashlink' in request.POST:
        try:
            hashlink = models.HashlinkDocument.objects.get(hashlink=request.POST['hashlink'])
        except models.HashlinkDocument.DoesNotExist:
            # TODO: Log
            return http.HttpResponseBadRequest()

        store_path(previous, hashlink.hashlink)

        # TODO: Remove
        print hashlink.state

        return http.HttpResponse(simplejson.dumps(hashlink.state), mimetype="application/json; charset=utf-8")

    else:
        # TODO: Log
        return http.HttpResponseBadRequest()
